package Gear::Remotes;
use strict;
use warnings;

our $VERSION = 0.008;
our $verbose;

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	@_,
    };
    bless($self,$class);
    die "fatal: . - not a gear repository.\n" if not -d '.git' or not -d '.gear' and not -f '.gear-rules';

    my $gear_remotes_file='.gear/upstream/remotes';
    $gear_remotes_file='.gear/upstream/remotes.'.$ENV{'USER'} if -e '.gear/upstream/remotes.'.$ENV{'USER'};
    my $suffix=$ENV{'GEAR_UPSTREAM_REMOTES_SUFFIX'};
    $gear_remotes_file='.gear/upstream/remotes'.$suffix if defined($suffix) and -e '.gear/upstream/remotes'.$suffix;

    my $gear_remotes_obj=Gear::Remotes::File->new(-file => $gear_remotes_file);
    my $git_remotes_obj=Gear::Remotes::File->new(-file => '.git/config');

    my %in_git=map{$_=>1} $git_remotes_obj->list();
    my (@entries_in_git,@entries_out_git);
    my (%svn_remotes,%git_remotes);
    foreach my $entry ($gear_remotes_obj->list()) {
	if ($in_git{$entry}) {
	    push @entries_in_git, $entry;
	} else {
	    push @entries_out_git, $entry;
	}
	if ($entry=~/^(svn-)?remote\.([^\.]+)\.url=(.*)/) {
	    my $svn=$1;
	    my $remotesname=$2;
	    my $url=$3;
	    $url=~s,^[^:]+://,,;
	    if ($url=~m,git(?:ery)?(\.alt(?:linux\.(?:org|ru)?)?)[:/],) {
		die "$gear_remotes_file: invalid remotes $entry: git.alt as upstream remotes makes no sense.\n";
	    } else {
		if ($svn) {
		    $svn_remotes{$remotesname}=1;
		} else {
		    $git_remotes{$remotesname}=1;
		}
	    }
	}
    }
    
    if (@entries_out_git and not @entries_in_git) {
	if ($self->{-autorestore}) {
	    `cat $gear_remotes_file >> .git/config`;
	} else {
	    die "Fatal: remotes listed in $gear_remotes_file are not present in .git/config. Run gear-restore-remotes first.\n";
	}
    } elsif (@entries_out_git and @entries_in_git) {
	print STDERR "entries in $gear_remotes_file and in .git/config\n";
	map {print STDERR "\t$_\n"} @entries_in_git;
	print STDERR "entries in $gear_remotes_file not in .git/config\n";
	map {print STDERR "\t$_\n"} @entries_out_git;
	die "Fatal: remotes listed in $gear_remotes_file conflicts with .git/config. Resolve conflict first.\n";
    }
    $self->{-git_remotes}=[sort keys (%git_remotes)];
    $self->{-svn_remotes}=[sort keys (%svn_remotes)];
    return $self;
}

sub fetch {
    my ($self)=@_;
    foreach my $remote (@{$self->{-svn_remotes}}) {
	print "git svn fetch $remote\n";
	system (qw/git svn fetch/,$remote)==0 or die "ERROR: git-svn fetch $remote failed\n";
    }

    foreach my $remote (@{$self->{-git_remotes}}) {
	print "git fetch $remote\n";
	system (qw/git fetch/,$remote)==0 or die "ERROR: git fetch $remote failed\n";
    }
}



package Gear::Remotes::File;

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-file => '.gear/upstream/remotes',
	@_,
    };
    bless($self,$class);
    die "Gear::Remotes: File ".$self->{-file}." does not exist.\n" if not -e $self->{-file};
    return $self;
}

sub list {
    my ($self)=@_;
    my @list;
    my $fn;
    open ($fn, "git config --file $self->{-file} --list|") || die "can't open $self->{-file}: $!";
    local $_;
    while (<$fn>) {
	chomp;
	push @list, $_;
    }
    close($fn);
    #$self->{-list}=\@list;
    return @list;
}


package Gear::Remotes::Helper;

use strict;
use warnings;

use IPC::Open2;

sub ls_git_tags_filtered {
    my $cmd='git tag';
    my $has_filter_tag=-x '.gear/upstream/filter-tag';
    my $has_transform_tag=-x '.gear/upstream/transform-tag';
    $cmd .= '| '.'.gear/upstream/filter-tag' if $has_filter_tag;
    open (my $fn, '-|',$cmd) or die "can't run $cmd: $!";
    my @git_tag;
    while (<$fn>) {
	chomp;
	next if not $has_filter_tag and not /^[Vv]?[\d\.]*\d(?:-rel|-release)?$/;
	push @git_tag, $_;
    }
    close($fn);
    my @git_tag_pairs;
    if (not $has_transform_tag) {
	foreach my $tag (@git_tag) {
	    my $ver=$tag;
	    $ver=~s/^[vV](\d)/$1/;
	    $ver=~s/(?:-rel|-release)$//;
	    push @git_tag_pairs, [$ver,$tag];
	}
    } else {
	my($chld_out, $chld_in, $pid);
	$pid = open2($chld_out, $chld_in, '.gear/upstream/transform-tag');
	print $chld_in map {"$_\n"} @git_tag;
	close($chld_in);
	foreach my $tag (@git_tag) {
	    my $ver=<$chld_out>;
	    chomp $ver;
	    push @git_tag_pairs, [$ver,$tag];
	}
	close($chld_out);
	waitpid( $pid, 0 );
	my $child_exit_status = $? >> 8;
    }
    return @git_tag_pairs;
}

# TODO: move Versort somewhere
# and made some Gear::Base that will get nvr and spec 
# without freaking out on tags and commits not in sync 
use RPM::uscan;
#use Gear::Rules;

sub watch_remotes {
    my ($action)=@_;
    my ($version_cmp, $watch_version, $newversion, $target)=&tagwatch_versions;
    if ($version_cmp>1) {
	print "Oops! local repository ($watch_version) is ahead of remote repository ($newversion from $target).\n";
    } elsif ($version_cmp==0) {
	print "Repository is up to date.\n";
    } else {
	print "New version $newversion is available from tag $target.\n";
	RPM::uscan::execute_user_action ($action, $watch_version, $newversion, $target) if $action;
    }
}

sub tagwatch_versions {
    my $pkg_dir=shift;
    $pkg_dir||='.';
    my $gear_remotes=Gear::Remotes->new();
    $gear_remotes->fetch();
    my @tag_pairs=Gear::Remotes::Helper::ls_git_tags_filtered();
    die "Gear::Remotes: no matching tags found\n" unless @tag_pairs;
    my @sorted = Devscripts::Versort::upstream_versort(@tag_pairs);
    my $ver=$sorted[0]->[0];
    #my $gear_rules=Gear::Rules->new();
    #my (undef,$uversion,undef)=$rules->get_nvr;
    my (undef, $upackage, $uversion) = RPM::uscan::altlinux_guess_package_and_version($pkg_dir);
    my $version_cmp=Devscripts::Versort::myvercmp($uversion, $ver);
    return ($version_cmp,$uversion, $ver,$sorted[0]->[1]);
}


1;

__END__
